<?php 

                   /**
                    * Creates the main widget for the simple links plugin
                    * @author mat lipe
                    * @since 1.3.14
                    * @uses registerd by init
                    * @uses the output can be filtered by using the 'simple_links_widget_output' filter
                    *       *   apply_filters( 'simple_links_widget_output', $output, $args );
                    *       the $args can be filtered by using the 'simple_links_widget_args' filter
                    *       *   apply_filters( 'simple_links_widget_args', $args );
                    *       the Widget Settings Can be filtered using the 'simple_links_widget_settings' filter
                    *       *   apply_filters( 'simple_links_widget_settings', $instance );
                    *       the Links object directly after get_posts()
                    *       *   apply_filters('simple_links_widget_links_object', $links, $instance, $args );
                    *       the links meta data one link at a time
                    *       *   apply_filters('simple_links_link_meta', get_post_meta($link->ID, false), $link, $instance, $args );
                    *   ** All Filters can be specified for a particular widget by ID
                    *      * e.g.   add_filter( 'simple_links_widget_settings_simple-links-3')
                    * 
                    * 
                    *
                    */
class SL_links_main extends WP_Widget {
    
    /**
     * Setup the Widget
     * @since 8/27/12
     */
    function __construct() {
        $widget_ops = array(
                'classname'   => 'sl-links-main',
                'description' => __('Displays a list of your Simple Links with options.', 'simple-links')
        );
        
        
        $control_ops = array(
                        'id_base' => 'simple-links',
                        'width'   => 305,
                        'height'  => 350,
            
        );

        $this->WP_Widget( 'simple-links', 'Simple Links', $widget_ops, $control_ops );

    }
    
    
    /**
     * Secret Method when outputing 2 columns and want them ordered alphabetical
     * 
     * @since 1.7.0
     * @uses add to the filter like so add_filter('simple_links_widget_links_object', array( 'SL_links_main', 'simpleLinksAdjust'), 1, 4 );
     * @uses currently just hanging out for future use
     * 
     * @TODO integrate this into core options
     */
    function twoColumns( $links_object, $instance, $args){
      $per_row = floor(count($links_object)/2);
      $count = 0;
      foreach( $links_object as $key => $l ){
        $count++;
          if( $count > $per_row ){
              $second[] = $l;
          } else {
              $first[] = $l;
          }
      }
      
      foreach( $first as $k => $l ){
        $new[] = $l;
            if( isset( $second[$k] ) ){
                $new[] = $second[$k]; 
            }  
      }
      
      return $new;
    }
    
    
    
    /**
     * Allows for migration widgets args from an old version of data to a new one
     * 
     * @uses run pre form and pre widget
     * 
     * @since 2.0
     */
    function migrateOldData($instance){
       global $simple_links; 


       if( isset( $instance['simple_links_version'] ) && ($instance['simple_links_version'] >= '2') ){
            return $instance;   
       }
        
        
       foreach( $simple_links->get_categories() as $cat ){
            if( isset( $instance[$cat]) && ($instance[$cat]) ){
                    $instance['category'][] = $cat;
                }
            }
            
            $instance['remove_line_break'] = $instance['line_break'];
            foreach( $simple_links->additional_fields as $field ){
                if( isset( $instance[$field]) && $instance[$field] ){
                    $instance['fields'][] = $field;
                }
            }
            

        return $instance;        
    }
    
    
    /**
     * The output of the widget to the site
     * @since 11.16.13
     * @see WP_Widget::widget()
     * @param $args the widget necessaties like $before_widget and $title
     * @param $instance all the settings for this particular widget
     * @uses See Class Docs for filtering the output,settings,and args
     * 
     * @see Notice error removed with help from WebEndev
     * @see nofollow error was remove with help from Heiko Manfrass
     */
    function widget( $args, $instance ) {
        
        do_action( 'simple_links_widget_pre_render', $args, $instance );
        
        extract( $args );

        //Filter for Changing the widget args
        $args = apply_filters('simple_links_widget_args', $args);
        $args = apply_filters('simple_links_widget_args_' . $widget_id, $args);
        
        
        //Call this filter to change the Widgets Settings Pre Compile
        $instance = apply_filters('simple_links_widget_settings', $instance);
        $instance = apply_filters('simple_links_widget_settings_' . $widget_id, $instance);


        //For any data which has not been resaved to the new structure
        $instance = $this->migrateOldData($instance);

        
    //--------------- Starts the Output --------------------------------------  
        
        $output .= $before_widget;
            //Add the title
            if( !empty( $instance['title'] ) ){
                $output .= $before_title. $instance['title'].$after_title;
            }
            
            $links = new SimpleLinksFactory($instance, 'widget');
            
            $output .= $links->output();
            
        //Close the Widget
        $output .= $after_widget;
        
        //The output can be filtered here
        $output = apply_filters( 'simple_links_widget_output_' . $widget_id, $output, $links->links, $instance, $args );
        echo apply_filters( 'simple_links_widget_output', $output, $links->links, $instance, $args );
    }
    
    
    
    
    /**
     * Updates the instance of each widget separately
     * @uses to make sure the data is valid
     * @see WP_Widget::update()
     * @since 9.21.13
     */
    function update( $new_instance, $old_instance ) {
        $new_instance['title'] = strip_tags( $new_instance['title'] );
        
        $new_instance = apply_filters('simple_links_widget_update', $new_instance, $this);       
 
        return $new_instance;
    
    }
    
    
    
    /**
     * Outputs the Widget form on the Widgets Page
     * @since 1.3.14
     * @see WP_Widget::form()
     */
    function form( $instance ) {
        global $simple_links;
 
        //backward compatibility - to allow for checkboxes to still be checked
        $instance = $this->migrateOldData($instance);

        ?>
        <input type="hidden" name="<?php echo $this->get_field_name( 'simple_links_version' ); ?>" value="<?php echo SIMPLE_LINKS_VERSION; ?>" />
        
        <em><?php _e('Be sure the see the Help Section in the Top Right Corner of the Screen for Questions!', 'simple-links');?></em><br><br>
        
        <strong><?php _e('Links Title', 'simple-links');?>:</strong>
        <input type="text" id="<?php echo $this->get_field_id( 'title' ); ?>" name="<?php echo $this->get_field_name( 'title' ); ?>" value="<?php 
        
        if( !isset( $instance['title']  ) ) $instance['title']  = '';
        echo esc_attr( $instance['title'] ); ?>" class="widefat" />
        
        <br><br>
        
        <strong><?php _e('Order Links By', 'simple-links');?></strong>
        <select id="<?php echo $this->get_field_id( 'orderby' ); ?>" name="<?php echo $this->get_field_name( 'orderby' ); ?>">
            <option value="menu_order" <?php selected($instance['orderby'],'menu_order'); ?>><?php _e('Link Order', 'simple-links');?></option>
            <option value="title" <?php selected($instance['orderby'],'title'); ?>><?php _e('Title', 'simple-links');?></option>
            <option value="rand" <?php selected($instance['orderby'],'rand'); ?>><?php _e('Random', 'simple-links');?></option>
        </select>
        
        <br><br>
        <strong><?php _e('Order', 'simple-links');?>:</strong>
        <select id="<?php echo $this->get_field_id( 'order' ); ?>" name="<?php echo $this->get_field_name( 'order' ); ?>">
            <option value="ASC" <?php selected($instance['order'],'ASC'); ?>><?php _e('Acending', 'simple-links');?></option>
            <option value="DESC" <?php selected($instance['order'],'DESC'); ?>><?php _e('Descending', 'simple-links');?></option>
        </select>
        
        <br><br>
       <strong><?php _e('Categories (optional)', 'simple-links');?>:</strong><br>
            <?php 
            
            foreach( $simple_links->get_categories() as $cat ){
                if( !isset( $instance['category'][$cat] ) ) $instance['category'][$cat] = 0;
                printf('&nbsp; &nbsp; <input class="cat" type="checkbox" value="%s" name="%s[%s]" %s/> %s <br>', $cat, $this->get_field_name('category'), $cat, checked($instance['category'][$cat], $cat, false), $cat );
            }
            ?>
       
       <br><br>
       <strong><?php _e('Number of Links', 'simple-links');?>:</strong>
            <select id="<?php echo $this->get_field_id( 'numberposts' ); ?>" name="<?php echo $this->get_field_name( 'numberposts' ); ?>">
                <option value="-1">All</option>
                <?php 
                for( $i = 1; $i<50; $i++){
                    printf('<option value="%s" %s>%s</option>', $i, selected($instance['numberposts'], $i ), $i );
                }
                ?>
            </select>

        <br><br>
       <strong><?php _e('Show Description', 'simple-links');?></strong> 
            <input type="checkbox" id="<?php echo $this->get_field_id( 'description' ); ?>" name="<?php echo $this->get_field_name( 'description' ); ?>" 
                    <?php 
                    
                    if( !isset( $instance['description']) ) $instance['description'] = 0;
                    checked($instance['description']); ?> value="1"/>
        
        
        <br><br>
        <strong><?php _e('Remove Line Break Between Image and Link', 'simple-links');?></strong> 
            <input type="checkbox" id="<?php echo $this->get_field_id( 'remove_line_break' ); ?>" name="<?php echo $this->get_field_name( 'remove_line_break' ); ?>" 
                    <?php 
                    
                    if( !isset( $instance['remove_line_break']) ) $instance['remove_line_break'] = 0;
                    checked($instance['remove_line_break']); ?> value="1"/>
        
        
        <br><br>
       <strong><?php _e('Show Image', 'simple-links');?></strong> 
            <input type="checkbox" id="<?php echo $this->get_field_id( 'show_image' ); ?>" name="<?php echo $this->get_field_name( 'show_image' ); ?>" 
                    <?php 
                    if( !isset( $instance['show_image']) ) $instance['show_image'] = 0;
                    checked($instance['show_image']); ?> value="1"/>
        
        
        <br><br>
        <strong><?php _e('Display Image Without Title', 'simple-links');?></strong> 
            <input type="checkbox" id="<?php echo $this->get_field_id( 'show_image_only' ); ?>" name="<?php echo $this->get_field_name( 'show_image_only' ); ?>" 
                    <?php 
                    if( !isset( $instance['show_image_only']) ) $instance['show_image_only'] = 0;
                    checked($instance['show_image_only']); ?> value="1"/>
        <br><br>
       <strong><?php _e('Image Size', 'simple-links');?>:</strong>
            <select id="<?php echo $this->get_field_id( 'image_size' ); ?>" name="<?php echo $this->get_field_name( 'image_size' ); ?>">
                <?php 
                foreach( $simple_links->image_sizes() as $size ){
                    printf('<option value="%s" %s>%s</option>', $size, selected($instance['image_size'], $size ), $size );
                }
                ?>
            </select>
        
        <br><br>
       <strong><?php _e('Include Additional Fields', 'simple-links');?>:</strong><br>
            <?php 
            
            $fields = $simple_links->getAdditionalFields();
            
            
            if( empty( $fields ) ){
                
                echo '<em>'.__('There have been no additional fields added', 'simple-links').'</em>';
                
            } else {

                foreach( $fields as $field ){
                    if( !isset( $instance['fields'][$field]) ) $instance['fields'][$field] = 0;
                    printf('&nbsp; &nbsp; <input class="cat" type="checkbox" value="%s" name="%s[%s]" %s/> %s <br>', $field, $this->get_field_name('fields'), $field, checked($instance['fields'][$field], $field, false), $field);
                }
            }
            ?>
            
        <br><br>    
        <strong><?php _e('Field Separator', 'simple-links');?>:</strong><br>
        <em><?php _e('HTML is allowed', 'simple-links');?>: - e.g. '&lt;br&gt;'</em><br>
        <input type="text" id="<?php echo $this->get_field_id( 'separator' ); ?>" name="<?php echo $this->get_field_name( 'separator' ); ?>" value="<?php 
        
        if( !isset( $instance['separator']  ) ) $instance['separator'] = '';
        echo esc_attr( $instance['separator'] ); ?>" class="widefat" />

        <?php 
        
        do_action('simple_links_widget_form', $instance, $this);
        
        
    }
    
    
    
    
    
    
}